@@ -12,11 +12,14 @@ use Farly::Director;
 use Farly::IPv4::Address;
 use Farly::IPv4::Network;
 use Farly::IPv4::Range;
+use Farly::IPv4::ICMPType;
 use Farly::Transport::Port;
+use Farly::Transport::PortGT;
+use Farly::Transport::PortLT;
 use Farly::Transport::PortRange;
 use Farly::Transport::Protocol;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 our ($volume,$dir,$file) = File::Spec->splitpath( $INC{'Farly.pm'} );
 Log::Log4perl::init( $volume.$dir.'Farly/Log/Farly.conf');
